@@ -1,5 +1,5 @@
-# $Id: TestSuite.pm 506 2014-07-04 18:07:33Z whynot $
-# Copyright 2009, 2010, 2014 Eric Pozharski <whynot@pozharski.name>
+# $Id: TestSuite.pm 526 2017-04-15 01:52:05Z sync $
+# Copyright 2009, 2010, 2014, 2017 Eric Pozharski <whynot@pozharski.name>
 # GNU LGPLv3
 # AS-IS, NO-WARRANTY, HOPE-TO-BE-USEFUL
 
@@ -10,7 +10,7 @@ package DB;
 sub get_fork_TTY { xterm_get_fork_TTY() }
 
 package t::TestSuite;
-use version 0.77; our $VERSION = version->declare( v0.1.8 );
+use version 0.77; our $VERSION = version->declare( v0.1.9 );
 # TODO:201406091242:whynot: Go B<parent()> whenever possible (perl v5.10.0 or something).
 # http://www.cpantesters.org/cpan/report/5947de70-df50-11e3-a498-53c68706f0e4
 use base   qw| Exporter |;
@@ -497,10 +497,12 @@ sub FAFTS_discover_lib ( ) {
   qw| LC_CTYPE LC_COLLATE LC_TIME LC_NUMERIC LC_MONETARY LC_MESSAGES
       LC_PAPER   LC_NAME   LC_ADDRESS   LC_TELEPHONE  LC_MEASUREMENT
       LC_IDENTIFICATION         LC_ALL         LANG         LANGUAGE |;
+# FIXME:201704081651:whynot: Disabling apt(8) for good.  JK abandoning.
+                                                                    return '';
     my $pid = open my $fh, q{-|}, @$aptconfig                    or return '';
-    while(my $line = <$fh>) {
+    while( my $line = <$fh> ) {
         $line =~ m{^Dir::Bin::methods\s+"(.+)";$}                     or next;
-        $lib = $1;      last }
+        $lib = $1;        last }
     undef                                                      while( <$fh> );
     close $fh                  or die qq|[apt-config]: close failed: $! ($?)|;
     waitpid $pid, 0;
