@@ -1,4 +1,4 @@
-# $Id: TestSuite.pm 501 2014-05-14 22:19:48Z whynot $
+# $Id: TestSuite.pm 505 2014-06-12 20:42:49Z whynot $
 # Copyright 2014 Eric Pozharski <whynot@pozharski.name>
 # GNU LGPLv3
 # AS-IS, NO-WARRANTY, HOPE-TO-BE-USEFUL
@@ -11,6 +11,7 @@ use version 0.77; our $VERSION = version->declare( v0.1.1 );
 
 use base qw| Exporter |;
 use Getopt::Long;
+use File::AptFetch::ConfigData;
 
 =head1 NAME
 
@@ -36,6 +37,38 @@ GetOptions \%main::opts, qw{ global|g=s% local|l=s% help version }      or die
 $main::opts{help}                    and die qq|[--help] isn't implemented\n|;
 $main::opts{version}                and die qq|[--version] isn't implemented|;
 
+=head2 Specially handled global options
+
+=over
+
+=cut
+
+=item I<--global timeout>
+
+Applies on I<timeout> option of C<F::AF::CD>.
+Default is C<10sec>, provided anyway.
+I<--local timeout> is ignored (unapplicable).
+
+=cut
+
+File::AptFetch::ConfigData->set_config( timeout =>
+  exists $main::opts{global}{timeout} ? $main::opts{global}{timeout} : 10 );
+
+=item I<--global tick>
+
+Applies on I<tick> option of C<F::AF::CD>.
+Default is C<1sec>, provided anyway.
+I<--local tick> is ignored (unapplicable).
+
+=cut
+
+File::AptFetch::ConfigData->set_config( tick =>
+  exists $main::opts{global}{tick} ? $main::opts{global}{tick} :  1 );
+
+=back
+
+=cut
+
 =head1 OVERRIDES
 
 Those are provided to make routines of B<t::TS> available without getting in
